﻿Type=Service
Version=2.00
StartAtBoot=False
@EndOfDesignText@
'Service module
Sub Process_Globals
	Dim WifiServer As ServerSocket
	Dim BluetoothServer As Serial
	Dim Client As Socket
	Dim Streams As AsyncStreams
	Dim Notification1 As Notification
	Dim PE As PhoneEvents
	Dim Phone As Phone
	Dim PhoneId As PhoneId
	Dim LogCat As LogCat
	Dim BC As ByteConverter
	Dim ConnectedStatus As Boolean
	Dim Out As OutputStream
	Dim LastToastMessageShown As Long
	Dim PING, APK_COPY_START, APK_PACKET, APK_COPY_CLOSE, SEND_DEVICE_NAME As Byte
	Dim START_LOGCAT, STOP_LOGCAT, LOGCAT_DATA As Byte
	Dim LAUNCH_PROGRAM, KILL_PROGRAM, ASK_FOR_DESIGNER As Byte
	PING = 1
	APK_COPY_START = 2
	APK_PACKET = 3
	APK_COPY_CLOSE = 4
	SEND_DEVICE_NAME = 5
	START_LOGCAT = 6
	STOP_LOGCAT = 7
	LOGCAT_DATA = 8
	LAUNCH_PROGRAM = 9
	KILL_PROGRAM = 10
	ASK_FOR_DESIGNER = 11
	Dim DisconnectTimer As Timer
	Dim DisconnectTicks As Int
	Dim wifi As Boolean
	Dim isBluetoothListening As Boolean
End Sub
Sub Service_Create
	WifiServer.Initialize(6789, "Server")
	BluetoothServer.Initialize("BluetoothServer")
	DisconnectTimer.Initialize("DisconnectTimer", 1000)
	Notification1.Initialize
	Notification1.Icon = "icon"
	Notification1.Light = False
	Notification1.Sound = False
	Notification1.Vibrate = False
	Notification1.SetInfo("B4A-Bridge", "Disconnected", Main)
	Service.StartForeground(1, Notification1)
	PE.Initialize("PE")
End Sub

Sub Service_Start
	Try
		UpdateStatus (ConnectedStatus, True)
	Catch
		ShowToastMessage("Error starting service: " & LastException.Message)
	End Try
End Sub
Sub UpdateStatus (Connected As Boolean, AllowFurtherConnections As Boolean)
	ConnectedStatus = Connected
	If Connected Then
		DisconnectTimer.Enabled = True
		DisconnectTicks = 0
		Notification1.SetInfo("B4A-Bridge", "Connected", Main)
		CheckIfDesignerIsInstalled
	Else
		If Streams.IsInitialized Then Streams.Close
		DisconnectTimer.Enabled = False
		Notification1.SetInfo("B4A-Bridge", "Disconnected", Main)
	End If
	isBluetoothListening = False
	If AllowFurtherConnections Then
		'Service should already be initialized here.
		Notification1.Notify(1)
		If wifi Then
			WifiServer.Listen
		Else
			Dim ba As BluetoothAdmin
			BluetoothServer.Listen2("b4a-bridge", "fa8fc0d0-afac-11de-8a39-08f0200c9a66")
			isBluetoothListening = True
		End If
	Else
		WifiServer.Close
		BluetoothServer.Disconnect
	End If
	CallSub(Main, "UpdateStatus")
End Sub
Sub DisconnectTimer_Tick
	DisconnectTicks = DisconnectTicks + 1
	If DisconnectTicks = 10 Then
		UpdateStatus(False, True)
	End If
End Sub
Sub Service_Destroy
	Service.StopForeground(1)
	UpdateStatus (False, False)
	LogCat.LogCatStop
End Sub
Sub BluetoothServer_Connected (Success As Boolean)
	If Streams.IsInitialized Then Streams.Close
	If Success Then
		Log("Connected to B4A-Bridge (Bluetooth)")
		Streams.InitializePrefix(BluetoothServer.InputStream, False, BluetoothServer.OutputStream, _
			"Streams")
		AfterConnect
	Else
		ShowToastMessage(LastException.Message)
		UpdateStatus(False, True)
	End If
End Sub
Sub Server_NewConnection (Successful As Boolean, NewSocket As Socket)
	If Streams.IsInitialized Then Streams.Close
	If Successful Then
		Log("Connected to B4A-Bridge (Wifi)")
		Client = NewSocket
		Streams.InitializePrefix(Client.InputStream, False, Client.OutputStream, _
			"Streams")
		AfterConnect
	Else
		ShowToastMessage(LastException.Message)
		UpdateStatus(False, True)
	End If
End Sub
Sub AfterConnect
	Dim pingB(1) As Byte
	pingB(0) = PING
	Streams.Write(pingB)
	Dim name As String
	name = Phone.Manufacturer & " " & Phone.Model & "-" & PhoneId.GetDeviceId
	Dim nameBytes() As Byte
	nameBytes = name.GetBytes("UTF8")
	Streams.Write(AddCommandToBytes(SEND_DEVICE_NAME, nameBytes, nameBytes.Length))
	UpdateStatus(True, True)
End Sub
Sub Streams_Error
	ShowToastMessage(LastException.Message)
	UpdateStatus(False, True)
End Sub
Sub Streams_Terminated
	UpdateStatus(False, True)
	Log("Streams_terminated")
End Sub


Sub AddCommandToBytes(Command As Byte, Buffer() As Byte, Length As Int) As Byte()
	Dim B(Length + 1) As Byte
	B(0) = Command
	BC.ArrayCopy(Buffer, 0, B, 1, Length)
	Return B
End Sub

Sub Streams_NewData (Buffer() As Byte)
	If Streams.IsInitialized = False Then Return
	DisconnectTicks = 0 'reset the disconnect timer
	Dim command As Byte
	command = Buffer(0)
	Select command
		Case PING
			Dim B(1) As Byte
			B(0) = PING
			Streams.Write(B)
		Case APK_COPY_START
			HandleFileStart
		Case APK_PACKET
			HandleFilePacket(Buffer)
		Case APK_COPY_CLOSE
			HandleFileClose
		Case START_LOGCAT
			StartLogcat(Buffer)
		Case STOP_LOGCAT
			StopLogcat
		Case LAUNCH_PROGRAM
			LaunchProgram(Buffer)
		Case KILL_PROGRAM
			KillProgram(Buffer)
	End Select
End Sub
Sub CheckIfDesignerIsInstalled
	Dim pm As PackageManager
	Dim packages As List
	packages = pm.GetInstalledPackages
	Dim buffer() As Byte
	If packages.IndexOf("anywheresoftware.b4a.designer") = -1 Then
		buffer = "0".GetBytes("UTF8")
	Else
		Dim s As String
		s = pm.GetVersionCode("anywheresoftware.b4a.designer")
		buffer = s.GetBytes("UTF8")
	End If
	
	'we are sending the installed designer version so the IDE will decide if a new version should be installed.
	Streams.Write(AddCommandToBytes(ASK_FOR_DESIGNER, buffer, buffer.Length))
End Sub
Sub LaunchProgram (Buffer() As Byte)
	Dim s As String
	s = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	Phone.Shell("am", Regex.Split(",", s), Null, Null)
End Sub
Sub KillProgram (Buffer() As Byte)
	Dim package As String
	package = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	Dim sb As StringBuilder
	sb.Initialize
	Phone.Shell("ps", Null, sb, Null)
	Dim m As Matcher
	m = Regex.Matcher2("^[^ ]*\s+(\d+) .*" & package, Regex.MULTILINE, sb.ToString)
	If m.Find Then
		Log("Package found: " & package)
		Phone.Shell("kill", Array As String(m.Group(1)), Null, Null)
	End If
End Sub
Sub StartLogcat (Buffer() As Byte)
	Dim args As String
	args = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	LogCat.LogCatStart(Regex.Split(",", args), "LogCat")
End Sub

Sub LogCat_LogCatData (Buffer() As Byte, Length As Int)
	If ConnectedStatus = True Then
		Streams.Write(AddCommandToBytes(LOGCAT_DATA, Buffer, Length))
	End If
End Sub
Sub StopLogcat
	LogCat.LogCatStop
End Sub
Sub HandleFileStart
	Log("Installing file.")
	If Out.IsInitialized Then Out.Close
	Out = File.OpenOutput(Main.folder, "temp.apk", False)
End Sub
Sub HandleFilePacket(Buffer() As Byte)
	Out.WriteBytes(Buffer, 1, Buffer.Length - 1)
End Sub
Sub HandleFileClose
	Out.Close
	If wifi = False Then
		File.WriteString(File.DirRootExternal, "b4a_bluetooth.txt", "0")
	Else
		File.Delete(File.DirRootExternal, "b4a_bluetooth.txt")
	End If
	'ask the system to install the apk
	Dim i As Intent
	i.Initialize(i.ACTION_VIEW, "file://" & File.Combine(Main.folder, "temp.apk"))
	i.SetType("application/vnd.android.package-archive")
	StartActivity(i)
End Sub
Sub PE_PackageAdded (Package As String, Intent As Intent)
	Log("PackageAdded: " & Package)
End Sub
Sub PE_ConnectivityChanged (NetworkType As String, State As String, Intent As Intent)
	If NetworkType = "WIFI" Then CallSub(Main, "UpdateIp")
End Sub
Sub ShowToastMessage(s As String)
	Log(s)
	If LastToastMessageShown + 5 * DateTime.TicksPerSecond > DateTime.Now Then
		Return
	Else
		LastToastMessageShown = DateTime.Now
		ToastMessageShow(s, True)
	End If
End Sub
